use actix_web::{get, App, HttpResponse, HttpServer};
use askama::Template;

use oidc_test_client::app_config::AppConfig;
use oidc_test_client::openid_primitives::OpenIDConfig;
use oidc_test_client::remote_ip::RemoteIP;
use oidc_test_client::state_manager::StateManager;

#[get("/assets/bootstrap.min.css")]
async fn bootstrap() -> HttpResponse {
    HttpResponse::Ok()
        .content_type("text/css")
        .body(include_str!("../assets/bootstrap.min.css"))
}

#[get("/assets/cover.css")]
async fn cover() -> HttpResponse {
    HttpResponse::Ok()
        .content_type("text/css")
        .body(include_str!("../assets/cover.css"))
}

#[derive(Template)]
#[template(path = "home.html")]
struct HomeTemplate {
    redirect_url: String,
}

#[get("/")]
async fn home() -> HttpResponse {
    HttpResponse::Ok().content_type("text/html").body(
        HomeTemplate {
            redirect_url: AppConfig::get().redirect_url(),
        }
        .render()
        .unwrap(),
    )
}

#[get("/start")]
async fn start(remote_ip: RemoteIP) -> HttpResponse {
    let config = OpenIDConfig::load_from(&AppConfig::get().configuration_url)
        .await
        .expect("Failed to load provider configuration!");

    let authorization_url = config.authorization_url(
        &AppConfig::get().client_id,
        &StateManager::gen_state(&remote_ip).expect("Failed to generate state!"),
        &AppConfig::get().redirect_url(),
    );

    HttpResponse::Found()
        .append_header(("Location", authorization_url))
        .finish()
}

#[actix_web::main]
async fn main() -> std::io::Result<()> {
    env_logger::init_from_env(env_logger::Env::new().default_filter_or("info"));

    log::info!("Init state manager");
    StateManager::init();

    log::info!("Will listen on {}", AppConfig::get().listen_addr);

    HttpServer::new(|| {
        App::new()
            .service(bootstrap)
            .service(cover)
            .service(home)
            .service(start)
    })
    .bind(&AppConfig::get().listen_addr)
    .expect("Failed to bind server!")
    .run()
    .await
}
